@@ -8,10 +8,18 @@ namespace acommon {
 class GlobalCacheBase;
 template <class Data> class GlobalCache;
 
+// get_cache_data (both versions) and release_cache_data will acquires
+// the cache's lock
+
 template <class Data>
 PosibErr<Data *> get_cache_data(GlobalCache<Data> *, 
                                 typename Data::CacheConfig *, 
                                 const typename Data::CacheKey &);
+template <class Data>
+PosibErr<Data *> get_cache_data(GlobalCache<Data> *, 
+                                typename Data::CacheConfig *, 
+                                typename Data::CacheConfig2 *, 
+                                const typename Data::CacheKey &);
 
 class Cacheable;
 void release_cache_data(GlobalCacheBase *, const Cacheable *);
@@ -27,9 +35,11 @@ public: // but don't use
   Cacheable * * prev;
   mutable int refcount;
   GlobalCacheBase * cache;
+public:
   bool attached() {return prev;}
-  void copy() const;
-  void release() const {release_cache_data(cache,this);}
+  void copy_no_lock() const {refcount++;}
+  void copy() const; // Acquires cache->lock
+  void release() const {release_cache_data(cache,this);} // Acquires cache->lock
   Cacheable(GlobalCacheBase * c = 0) : next(0), prev(0), refcount(1), cache(c) {}
   virtual ~Cacheable() {}
 };
@@ -44,7 +54,7 @@ public:
     if (ptr) ptr->release();
     ptr = p;
   }
-  void copy(Data * p) {p->copy(); reset(p);}
+  void copy(Data * p) {if (p) p->copy(); reset(p);}
   Data * release() {Data * tmp = ptr; ptr = 0; return tmp;}
 
   Data & operator*  () const {return *ptr;}
@@ -53,22 +63,24 @@ public:
   operator Data * ()   const {return ptr;}
 
   CachePtr() : ptr(0) {}
-  CachePtr(const CachePtr & other) {ptr = other.ptr; ptr->copy();}
+  CachePtr(const CachePtr & other) {ptr = other.ptr; if (ptr) ptr->copy();}
   void operator=(const CachePtr & other) {copy(other.ptr);}
   ~CachePtr() {reset(0);}
-
-  PosibErr<void> setup(GlobalCache<Data> * cache, 
-                       typename Data::CacheConfig * config, 
-                       const typename Data::CacheKey & key) {
-    PosibErr<Data *> pe = get_cache_data(cache, config, key);
-    if (pe.has_err()) return pe;
-    reset(pe.data);
-    return no_err;
-  }
-
-  void reset_cache(const char * = 0);
 };
 
+template <class Data>
+PosibErr<void> setup(CachePtr<Data> & res,
+                     GlobalCache<Data> * cache, 
+                     typename Data::CacheConfig * config, 
+                     const typename Data::CacheKey & key) {
+  PosibErr<Data *> pe = get_cache_data(cache, config, key);
+  if (pe.has_err()) return pe;
+  res.reset(pe.data);
+  return no_err;
+}
+
+bool reset_cache(const char * = 0);
+
 }
 
 #endif
