@@ -83,7 +83,7 @@ namespace acommon {
 		 const KeyInfo * mainbegin, 
 		 const KeyInfo * mainend)
     : name_(name)
-    , first_(0), insert_point_(&first_), others_(0)
+    , first_(0), insert_point_(&first_)
     , committed_(true), attached_(false)
     , md_info_list_index(-1)
     , settings_read_in_(false)
@@ -122,9 +122,6 @@ namespace acommon {
 
   void Config::copy(const Config & other)
   {
-    assert(other.others_ == 0);
-    others_ = 0;
-
     name_ = other.name_;
 
     committed_ = other.committed_;
@@ -180,12 +177,6 @@ namespace acommon {
       first_ = tmp;
     }
 
-    while (others_) {
-      Entry * tmp = others_->next;
-      delete first_;
-      others_ = tmp;
-    }
-
     Vector<Notifier *>::iterator i   = notifier_list.begin();
     Vector<Notifier *>::iterator end = notifier_list.end();
 
@@ -333,6 +324,16 @@ namespace acommon {
     return cur ? cur->value : get_default(ki);
   }
 
+  PosibErr<Config::Value> Config::retrieve_value(ParmStr key) const
+  {
+    RET_ON_ERR_SET(keyinfo(key), const KeyInfo *, ki);
+    if (ki->type == KeyInfoList) return make_err(key_not_string, ki->name);
+
+    const Entry * cur = lookup(ki->name);
+
+    return cur ? Value(cur->value,cur->secure) : Value(get_default(ki), true);
+  }
+  
   PosibErr<String> Config::retrieve_any(ParmStr key) const
   {
     RET_ON_ERR_SET(keyinfo(key), const KeyInfo *, ki);
@@ -383,9 +384,12 @@ namespace acommon {
     return atoi(value.str());
   }
 
-  void Config::lookup_list(const KeyInfo * ki,
-                           MutableContainer & m,
-                           bool include_default) const
+#define RET_ON_ERR_WRAP(prefix, key, cmd)                                \
+  do{PosibErrBase pe(cmd);if(pe.has_err())return pe.with_key(prefix, strncmp(key, "f-", 2) == 0 ? key + 2 : key);}while(false)
+
+  PosibErr<void> Config::lookup_list(const KeyInfo * ki,
+                                     MutableContainer & m,
+                                     bool include_default) const
   {
     const Entry * cur = first_;
     const Entry * first_to_use = 0;
@@ -415,7 +419,7 @@ namespace acommon {
 
     if (cur && cur->action == Set) {
       if (!include_default) m.clear();
-      m.add(cur->value);
+      RET_ON_ERR_WRAP("", ki->name, m.add(cur->value));
       cur = cur->next;
     }
 
@@ -427,21 +431,24 @@ namespace acommon {
     while (cur) {
       if (cur->key == ki->name) {
         if (cur->action == ListAdd)
-          m.add(cur->value);
+          RET_ON_ERR_WRAP("add-", ki->name, m.add(cur->value));
         else if (cur->action == ListRemove)
-          m.remove(cur->value);
+          RET_ON_ERR_WRAP("remove-", ki->name, m.remove(cur->value));
       }
       cur = cur->next;
     }
+    return no_err;
   }
 
+#undef RET_ON_ERR_WRAP
+
   PosibErr<void> Config::retrieve_list(ParmStr key, 
 				       MutableContainer * m) const
   {
     RET_ON_ERR_SET(keyinfo(key), const KeyInfo *, ki);
     if (ki->type != KeyInfoList) return make_err(key_not_list, ki->name);
 
-    lookup_list(ki, *m, true);
+    RET_ON_ERR(lookup_list(ki, *m, true));
 
     return no_err;
   }
@@ -763,7 +770,7 @@ namespace acommon {
       }
       res.append(':');
     }
-    if (res.back() == ':') res.pop_back();
+    if (!res.empty() && res.back() == ':') res.pop_back();
   }
 
   struct ListAddHelper : public AddableContainer 
@@ -798,6 +805,7 @@ namespace acommon {
     Entry * entry = new Entry;
     entry->key = key;
     entry->value = value;
+    entry->secure = true;
     return set(entry);
   }
   
@@ -863,7 +871,7 @@ namespace acommon {
       Entry * ent = new Entry;
       ent->key = entry->key;
       ent->action = ListClear;
-      set(ent);
+      RET_ON_ERR(set(ent));
 
       ListAddHelper helper;
       helper.config = this;
@@ -876,24 +884,23 @@ namespace acommon {
 
   PosibErr<void> Config::merge(const Config & other)
   {
-    const Entry * src  = other.first_;
-    while (src) 
+    for (const Entry * src  = other.first_; src; src = src->next)
     {
+      if (src->action == NoOp) continue;
       Entry * entry = new Entry(*src);
       entry->next = *insert_point_;
       *insert_point_ = entry;
       insert_point_ = &entry->next;
       if (committed_) RET_ON_ERR(commit(entry));
-      src = src->next;
     }
     return no_err;
   }
 
-  void Config::lang_config_merge(const Config & other,
-                                 int which, ParmStr data_encoding)
+  PosibErr<void> Config::lang_config_merge(const Config & other,
+                                           int which, ParmStr data_encoding)
   {
     Conv to_utf8;
-    to_utf8.setup(*this, data_encoding, "utf-8", NormTo);
+    RET_ON_ERR(to_utf8.setup(*this, data_encoding, "utf-8", NormTo));
     const Entry * src  = other.first_;
     Entry * * ip = &first_;
     while (src)
@@ -910,6 +917,7 @@ namespace acommon {
       }
       src = src->next;
     }
+    return no_err;
   }
 
 
@@ -950,9 +958,11 @@ namespace acommon {
         pe = make_err(key_not_list, entry->key);
         goto error;
       }
-      
-      assert(ki->def != 0); // if null this key should never have values
-      // directly added to it
+
+      if (!ki->def) // if null this key should never have values
+                    // directly added to it
+        return make_err(aerror_cant_change_value, entry->key);
+
       String value(entry->action == Reset ? get_default(ki) : entry->value);
       
       switch (ki->type) {
@@ -1287,9 +1297,9 @@ namespace acommon {
       }
       
       {
-      insert_point_ = &first_;
-      PosibErrBase pe = read_in_file(retrieve("per-conf-path"));
-      if (pe.has_err() && !pe.has_err(cant_read_file)) return pe;
+        insert_point_ = &first_;
+        PosibErrBase pe = read_in_file(retrieve("per-conf-path"));
+        if (pe.has_err() && !pe.has_err(cant_read_file)) return pe;
       }
       
       {
@@ -1310,22 +1320,32 @@ namespace acommon {
   PosibErr<void> Config::commit_all(Vector<int> * phs, const char * codeset)
   {
     committed_ = true;
-    others_ = first_;
+    Entry * uncommited = first_;
     first_ = 0;
     insert_point_ = &first_;
     Conv to_utf8;
     if (codeset)
       RET_ON_ERR(to_utf8.setup(*this, codeset, "utf-8", NormTo));
-    while (others_) {
-      *insert_point_ = others_;
-      others_ = others_->next;
-      (*insert_point_)->next = 0;
-      RET_ON_ERR_SET(commit(*insert_point_, codeset ? &to_utf8 : 0), int, place_holder);
+    PosibErr<void> ret;
+    while (uncommited) {
+      Entry * cur = uncommited;
+      uncommited = cur->next;
+      cur->next = 0;
+      *insert_point_ = cur;
+      insert_point_ = &((*insert_point_)->next);
+      PosibErr<int> pe = commit(cur, codeset ? &to_utf8 : 0);
+      if (pe.has_err()) {
+        if (ret.has_err())
+          pe.ignore_err();
+        else
+          ret = pe;
+        continue;
+      }
+      int place_holder = pe.data;
       if (phs && place_holder != -1 && (phs->empty() || phs->back() != place_holder))
         phs->push_back(place_holder);
-      insert_point_ = &((*insert_point_)->next);
     }
-    return no_err;
+    return ret;
   }
 
   PosibErr<void> Config::set_committed_state(bool val) {
@@ -1354,7 +1374,7 @@ namespace acommon {
     {"actual-dict-dir", KeyInfoString, "<dict-dir^master>", 0}
     , {"actual-lang",     KeyInfoString, "", 0} 
     , {"conf",     KeyInfoString, "aspell.conf",
-       /* TRANSLATORS: The remaing strings in config.cpp should be kept
+       /* TRANSLATORS: The remaining strings in config.cpp should be kept
           under 50 characters, begin with a lower case character and not
           include any trailing punctuation marks. */
        N_("main configuration file")}
@@ -1379,6 +1399,8 @@ namespace acommon {
        N_("filter mode"), KEYINFO_COMMON}
     , {"extra-dicts", KeyInfoList, "",
        N_("extra dictionaries to use")}
+    , {"wordlists", KeyInfoList, "",
+       N_("files with list of extra words to accept")}
     , {"home-dir", KeyInfoString, HOME_DIR,
        N_("location for personal files")}
     , {"ignore",   KeyInfoInt   , "1",
@@ -1435,6 +1457,8 @@ namespace acommon {
        N_("maximum number that can be strung together"), KEYINFO_MAY_CHANGE}
     , {"run-together-min",    KeyInfoInt,   "3",
        N_("minimal length of interior words"), KEYINFO_MAY_CHANGE}
+    , {"camel-case", KeyInfoBool,  "false",
+       N_("consider camel case words legal"), KEYINFO_MAY_CHANGE}
     , {"save-repl", KeyInfoBool  , "true",
        N_("save replacement pairs on save all")}
     , {"set-prefix", KeyInfoBool, "true",
@@ -1445,11 +1469,9 @@ namespace acommon {
        N_("no longer used"), KEYINFO_HIDDEN}
     , {"sug-mode",   KeyInfoString, "normal",
        N_("suggestion mode"), KEYINFO_MAY_CHANGE | KEYINFO_COMMON}
-    , {"sug-edit-dist", KeyInfoInt, "1",
+    , {"sug-typo-analysis", KeyInfoBool, "true",
        /* TRANSLATORS: "sug-mode" is a literal value and should not be
           translated. */
-       N_("edit distance to use, override sug-mode default")}
-    , {"sug-typo-analysis", KeyInfoBool, "true",
        N_("use typo analysis, override sug-mode default")}
     , {"sug-repl-table", KeyInfoBool, "true",
        N_("use replacement tables, override sug-mode default")}
