@@ -6,46 +6,42 @@
 #include "suggest.hpp"
 #include "editdist.hpp"
 #include "typo_editdist.hpp"
+#include "cache.hpp"
 
 namespace aspeller {
   class Speller;
+  class SpellerImpl;
   class Suggest;
 
   struct SuggestParms {
     // implementation at the end of suggest.cc
 
     EditDistanceWeights     edit_distance_weights;
-    TypoEditDistanceWeights typo_edit_distance_weights;
+    CachePtr<const TypoEditDistanceInfo> ti;
+
+    bool try_one_edit_word, try_scan_0, try_scan_1, try_scan_2, try_ngram;
+
+    int ngram_threshold, ngram_keep;
+
+    bool check_after_one_edit_word;
 
     bool use_typo_analysis;
     bool use_repl_table;
 
-    int normal_soundslike_weight; // percentage
-
-    int small_word_soundslike_weight; 
-    int small_word_threshold;
-    
     int soundslike_weight;
     int word_weight;
 
-    int soundslike_level; // either 1 or 2
-    
     int skip;
     int span;
     int limit;
 
     String split_chars;
+    bool camel_case;
 
-    SuggestParms(ParmString mode = "normal") {
-      set(mode);
-    }
-    
-    PosibErr<void> set(ParmString mode = "normal");
-    PosibErr<void> fill_distance_lookup(const Config * c, const Language & l);
-    
-    virtual ~SuggestParms() {}
-    virtual SuggestParms * clone() const;
-    virtual void set_original_word_size(int size);
+    SuggestParms() {}
+
+    PosibErr<void> init(ParmString mode, SpellerImpl * sp);
+    PosibErr<void> init(ParmString mode, SpellerImpl * sp, Config *);
   };
   
   Suggest * new_default_suggest(const Speller *, const SuggestParms &);
