@@ -16,7 +16,8 @@ namespace aspeller {
   class Soundslike {
   public:
     virtual String soundslike_chars() const = 0;
-    virtual String to_soundslike(ParmString) const = 0;
+    // string must be null terminated even if len is given
+    virtual char * to_soundslike(char *, const char *, int len = -1) const = 0;
     virtual const char * name() const = 0;
     virtual const char * version() const = 0;
     virtual PosibErr<void> setup(Conv &) = 0;
