@@ -1,5 +1,5 @@
 // This file is part of The New Aspell
-// Copyright (C) 2000-2001 by Kevin Atkinson under the GNU LGPL
+// Copyright (C) 2000-2001,2011 by Kevin Atkinson under the GNU LGPL
 // license version 2.0 or 2.1.  You should have received a copy of the
 // LGPL license along with this library if you did not you can find it
 // at http://www.gnu.org/.
@@ -19,8 +19,11 @@
 #include "tokenizer.hpp"
 #include "convert.hpp"
 #include "stack_ptr.hpp"
+#include "istream_enumeration.hpp"
 
-#include "iostream.hpp"
+//#include "iostream.hpp"
+
+#include "gettext.h"
 
 namespace aspeller {
   //
@@ -81,20 +84,28 @@ namespace aspeller {
     String cor1, cor2;
     String buf;
     bool correct = false;
-    if (pos = cor.find(' '), pos == String::npos) {
+    pos = cor.find(' '); 
+    if (pos == String::npos) {
       cor1 = cor;
       correct = check_affix(cor, w1, 0);
     } else {
       cor1 = (String)cor.substr(0,pos);
-      cor2 = (String)cor.substr(pos+1);
+      ++pos;
+      while (pos < cor.size() && cor[pos] == ' ') ++pos;
+      cor2 = (String)cor.substr(pos);
       correct = check_affix(cor1, w1, 0) && check_affix(cor2, w2, 0);
     }
     if (correct) {
       String cor_orignal_casing(cor1);
       if (!cor2.empty()) {
-        cor_orignal_casing += cor[pos];
+        cor_orignal_casing += cor[pos-1];
         cor_orignal_casing += cor2;
       }
+      // Don't try to add the empty string, causes all kinds of
+      // problems.  Can happen if the original replacement nothing but
+      // whitespace.
+      if (cor_orignal_casing.empty()) 
+        return no_err;
       if (first_word == 0 || cor != first_word) {
         lang().to_lower(buf, mis.str());
         repl_->add_repl(buf, cor_orignal_casing);
@@ -153,9 +164,9 @@ namespace aspeller {
     return false;
   }
 
-  inline bool SpellerImpl::check2(char * word, /* it WILL modify word */
-                                  bool try_uppercase,
-                                  CheckInfo & ci, GuessInfo * gi)
+  inline bool SpellerImpl::check_single(char * word, /* it WILL modify word */
+                                        bool try_uppercase,
+                                        CheckInfo & ci, GuessInfo * gi)
   {
     bool res = check_affix(word, ci, gi);
     if (res) return true;
@@ -168,18 +179,18 @@ namespace aspeller {
     return false;
   }
 
-  PosibErr<bool> SpellerImpl::check(char * word, char * word_end, 
-                                    /* it WILL modify word */
-                                    bool try_uppercase,
-                                    unsigned run_together_limit,
-                                    CheckInfo * ci, GuessInfo * gi)
+  CheckInfo * SpellerImpl::check_runtogether(char * word, char * word_end, 
+                                             /* it WILL modify word */
+                                             bool try_uppercase,
+                                             unsigned run_together_limit,
+                                             CheckInfo * ci, CheckInfo * ci_end,
+                                             GuessInfo * gi)
   {
-    assert(run_together_limit <= 8); // otherwise it will go above the 
-                                     // bounds of the word array
+    if (ci >= ci_end) return NULL;
     clear_check_info(*ci);
-    bool res = check2(word, try_uppercase, *ci, gi);
-    if (res) return true;
-    if (run_together_limit <= 1) return false;
+    bool res = check_single(word, try_uppercase, *ci, gi);
+    if (res) return ci;
+    if (run_together_limit <= 1) return NULL;
     enum {Yes, No, Unknown} is_title = try_uppercase ? Yes : Unknown;
     for (char * i = word + run_together_min_; 
          i <= word_end - run_together_min_;
@@ -187,19 +198,111 @@ namespace aspeller {
     {
       char t = *i;
       *i = '\0';
-      //FIXME: clear ci, gi?
-      res = check2(word, try_uppercase, *ci, gi);
+      clear_check_info(*ci);
+      res = check_single(word, try_uppercase, *ci, gi);
       if (!res) {*i = t; continue;}
       if (is_title == Unknown)
         is_title = lang_->case_pattern(word) == FirstUpper ? Yes : No;
       *i = t;
-      if (check(i, word_end, is_title == Yes, run_together_limit - 1, ci + 1, 0)) {
+      CheckInfo * ci_last = check_runtogether(i, word_end, is_title == Yes, run_together_limit - 1, ci + 1, ci_end, 0);
+      if (ci_last) {
         ci->compound = true;
         ci->next = ci + 1;
-        return true;
+        return ci_last;
       }
     }
-    return false;
+    return NULL;
+  }
+
+  PosibErr<bool> SpellerImpl::check(char * word, char * word_end, 
+                                    /* it WILL modify word */
+                                    bool try_uppercase,
+                                    unsigned run_together_limit,
+                                    CheckInfo * ci, CheckInfo * ci_end,
+                                    GuessInfo * gi, CompoundInfo * cpi)
+  {
+    clear_check_info(*ci);
+    bool res = check_runtogether(word, word_end, try_uppercase, run_together_limit, ci, ci_end, gi);
+    if (res) return true;
+    
+    CompoundWord cw = lang_->split_word(word, word_end - word, camel_case_);
+    if (cw.single()) return false;
+    bool ok = true;
+    CheckInfo * ci_prev = NULL;
+    do {
+      unsigned len = cw.word_len();
+      
+      char save = word[len];
+      word[len] = '\0';
+      CheckInfo * ci_last = check_runtogether(word, word + len, try_uppercase, run_together_limit, ci, ci_end, gi);
+      bool found = ci_last;
+      word[len] = save;
+
+      if (!found) {
+        if (cpi) {
+          ci_last = ci;
+          ok = false;
+          ci->word.str = word;
+          ci->word.len = len;
+          ci->incorrect = true;
+          cpi->incorrect_count++;
+          if (!cpi->first_incorrect)
+            cpi->first_incorrect = ci;
+        } else {
+          return false;
+        }
+      }
+
+      if (cpi)
+        cpi->count++;
+      
+      if (ci_prev) {
+        ci_prev->compound = true;
+        ci_prev->next = ci;
+      }
+
+      ci_prev = ci_last;
+      ci = ci_last + 1;
+      if (ci >= ci_end) {
+        if (cpi) cpi->count = 0;
+        return false;
+      }
+      
+      word = word + cw.rest_offset();
+      cw = lang_->split_word(cw.rest, cw.rest_len(), camel_case_);
+      
+    } while (!cw.empty());
+    
+    return ok;
+    
+    // for (;;) {
+    //   cw = lang_->split_word(cw.rest, cw.rest_len(), camel_case_);
+    //   if (cw.empty()) break;
+    //   if (ci + 1 >= ci_end) {
+    //     if (cpi) cpi->count = 0;
+    //     return false;
+    //   }
+    //   if (cpi) cpi->count++;
+    //   len = cw.word_len();
+    //   save = word[len];
+    //   word[len] = '\0';
+    //   ci_last = check_runtogether(word, word + len, try_uppercase, run_together_limit, ci + 1, ci_end, gi);
+    //   word[len] = save;
+    //   ci->compound = true;
+    //   ci->next = ci + 1;
+    //   if (ci_last) {
+    //     ci = ci_last;
+    //   } else if (cpi) {
+    //     ok = false;
+    //     ci->word.str = word;
+    //     ci->word.len = len;
+    //     ci->incorrect = true;
+    //     cpi->incorrect_count++;
+    //   } else {
+    //     return false;
+    //   }
+    //   word = word + cw.rest_offset();
+    // }
   }
 
   //////////////////////////////////////////////////////////////////////
@@ -226,15 +329,24 @@ namespace aspeller {
   }
 
   PosibErr<const WordList *> SpellerImpl::personal_word_list() const {
-    return static_cast<const WordList *>(personal_);
+    const WordList * wl = static_cast<const WordList *>(personal_);
+    if (!wl) return make_err(operation_not_supported_error, 
+                             _("The personal word list is unavailable."));
+    return wl;
   }
 
   PosibErr<const WordList *> SpellerImpl::session_word_list() const {
-    return static_cast<const WordList *>(session_);
+    const WordList * wl = static_cast<const WordList *>(session_);
+    if (!wl) return make_err(operation_not_supported_error, 
+                             _("The session word list is unavailable."));
+    return wl;
   }
 
   PosibErr<const WordList *> SpellerImpl::main_word_list() const {
-    return dynamic_cast<const WordList *>(main_);
+    const WordList * wl = dynamic_cast<const WordList *>(main_);
+    if (!wl) return make_err(operation_not_supported_error, 
+                             _("The main word list is unavailable."));
+    return wl;
   }
 
   const SpellerDict * SpellerImpl::locate (const Dict::Id & id) const
@@ -386,7 +498,10 @@ namespace aspeller {
       m->run_together_min_ = value;
       return no_err;
     }
-    
+    static PosibErr<void> camel_case(SpellerImpl * m, bool value) {
+      m->camel_case_ = value;
+      return no_err;
+    }
   };
 
   static UpdateMember update_members[] = 
@@ -406,6 +521,9 @@ namespace aspeller {
     ,{"run-together-min",  
         UpdateMember::Int,    
         UpdateMember::CN::run_together_min}
+    ,{"camel-case",
+        UpdateMember::Bool,    
+        UpdateMember::CN::camel_case}
   };
 
   template <typename T>
@@ -490,7 +608,7 @@ namespace aspeller {
     if (use_other_dicts && !personal_)
     {
       Dictionary * temp;
-      temp = new_default_writable_dict();
+      temp = new_default_writable_dict(*config_);
       PosibErrBase pe = temp->load(config_->retrieve("personal-path"),*config_);
       if (pe.has_err(cant_read_file))
         temp->set_check_lang(lang_name(), *config_);
@@ -502,14 +620,14 @@ namespace aspeller {
     if (use_other_dicts && !session_)
     {
       Dictionary * temp;
-      temp = new_default_writable_dict();
+      temp = new_default_writable_dict(*config_);
       temp->set_check_lang(lang_name(), *config_);
       RET_ON_ERR(add_dict(new SpellerDict(temp, *config_, session_id)));
     }
      
     if (use_other_dicts && !repl_)
     {
-      ReplacementDict * temp = new_default_replacement_dict();
+      ReplacementDict * temp = new_default_replacement_dict(*config_);
       PosibErrBase pe = temp->load(config_->retrieve("repl-path"),*config_);
       if (pe.has_err(cant_read_file))
         temp->set_check_lang(lang_name(), *config_);
@@ -518,9 +636,38 @@ namespace aspeller {
       RET_ON_ERR(add_dict(new SpellerDict(temp, *config_, personal_repl_id)));
     }
 
+    StringList wordlist_files;
+    config_->retrieve_list("wordlists", &wordlist_files);
+    if (!wordlist_files.empty()) {
+      Dictionary * dict = session_;
+      if (!dict) {
+        dict = new_default_writable_dict(*config_);
+        dict->set_check_lang(lang_name(), *config_);
+        RET_ON_ERR(add_dict(new SpellerDict(dict, *config_)));
+      }
+      const char * fn;
+      StringListEnumeration els = wordlist_files.elements_obj();
+      while ( (fn = els.next()) != 0) {
+        FStream f;
+        RET_ON_ERR(f.open(fn, "r"));
+        IstreamEnumeration els(f);
+        WordListIterator wl_itr(&els, lang_, 0);
+        wl_itr.init_plain(*config_);
+        for (;;) {
+          PosibErr<bool> pe = wl_itr.adv();
+          if (pe.has_err())
+            return pe.with_file(fn);
+          if (!pe.data) break;
+          PosibErr<void> pev = dict->add(wl_itr->word);
+          if (pev.has_err())
+            return pev.with_file(fn);
+        }
+      }
+    }
+
     const char * sys_enc = lang_->charmap();
-    String user_enc = config_->retrieve("encoding");
-    if (user_enc == "none") {
+    ConfigConvKey user_enc = config_->retrieve_value("encoding");
+    if (user_enc.val == "none") {
       config_->replace("encoding", sys_enc);
       user_enc = sys_enc;
     }
@@ -543,6 +690,8 @@ namespace aspeller {
     }
     run_together_min_    = config_->retrieve_int("run-together-min");
 
+    camel_case_ = config_->retrieve_bool("camel-case");
+
     config_->add_notifier(new ConfigNotifier(this));
 
     config_->set_attached(true);
