@@ -102,10 +102,13 @@ protected:
   
   String compatibility_file_name;
     
-  WritableBase(BasicType t, const char * n, const char * s, const char * cs)
+  WritableBase(BasicType t, const char * n, const char * s, const char * cs, const Config & cfg)
     : Dictionary(t,n),
       suffix(s), compatibility_suffix(cs),
-      use_soundslike(true) {fast_lookup = true;}
+      use_soundslike(true) {
+    fast_lookup = true;
+    validate_words = cfg.retrieve_bool("validate-words");
+  }
   virtual ~WritableBase() {}
   
   virtual PosibErr<void> save(FStream &, ParmString) = 0;
@@ -160,7 +163,8 @@ PosibErr<void> WritableBase::load(ParmString f0, Config & config,
     // ^^ FIXME 
     RET_ON_ERR(merge(in, f, &config));
       
-  } else if (f.substr(f.size()-suffix.size(),suffix.size()) 
+  } else if (f.size() >= suffix.size() &&
+             f.substr(f.size()-suffix.size(),suffix.size())
              == suffix) {
       
     compatibility_file_name = f.substr(0,f.size() - suffix.size());
@@ -186,8 +190,8 @@ PosibErr<void> WritableBase::load(ParmString f0, Config & config,
 PosibErr<void> WritableBase::merge(ParmString f0) {
   FStream in;
   Dict::FileName fn(f0);
-  RET_ON_ERR(open_file_readlock(in, fn.path));
-  RET_ON_ERR(merge(in, fn.path));
+  RET_ON_ERR(open_file_readlock(in, fn.path()));
+  RET_ON_ERR(merge(in, fn.path()));
   return no_err;
 }
 
@@ -383,7 +387,8 @@ public: //but don't use
 
 public:
 
-  WritableDict() : WritableBase(basic_dict, "WritableDict", ".pws", ".per") {}
+  WritableDict(const Config & cfg)
+    : WritableBase(basic_dict, "WritableDict", ".pws", ".per", cfg) {}
 
   Size   size()     const;
   bool   empty()    const;
@@ -499,7 +504,10 @@ WritableDict::Enum * WritableDict::detailed_elements() const {
 }
 
 PosibErr<void> WritableDict::add(ParmString w, ParmString s) {
-  RET_ON_ERR(check_if_valid(*lang(),w));
+  if (validate_words)
+    RET_ON_ERR(check_if_valid(*lang(),w));
+  else
+    RET_ON_ERR(check_if_sane(*lang(),w));
   SensitiveCompare c(lang());
   WordEntry we;
   if (WritableDict::lookup(w,&c,we)) return no_err;
@@ -570,16 +578,27 @@ PosibErr<void> WritableDict::merge(FStream & in,
   return no_err;
 }
 
+struct CStrLess {
+  bool operator() (const char * x, const char * y) const {
+    return strcmp(x, y) < 0;
+  }
+};
+
 PosibErr<void> WritableDict::save(FStream & out, ParmString file_name) 
 {
   out.printf("personal_ws-1.1 %s %i %s\n", 
              lang_name(), word_lookup->size(), file_encoding.c_str());
 
-  WordLookup::const_iterator i = word_lookup->begin();
-  WordLookup::const_iterator e = word_lookup->end();
+  Vector<const char *> words;
+  words.reserve(word_lookup->size());
+  for (WordLookup::const_iterator i = word_lookup->begin(), e = word_lookup->end();
+       i != e; ++i)
+    words.push_back(*i);
+  std::sort(words.begin(), words.end(), CStrLess());
     
   ConvP conv(oconv);
-  for (;i != e; ++i) {
+  for (Vector<const char *>::const_iterator i = words.begin(), e = words.end();
+       i != e; ++i) {
     write_n_escape(out, conv(*i));
     out << '\n';
   }
@@ -602,7 +621,8 @@ class WritableReplDict : public WritableBase
   WritableReplDict& operator=(const WritableReplDict&);
 
 public:
-  WritableReplDict() : WritableBase(replacement_dict, "WritableReplDict", ".prepl",".rpl") 
+  WritableReplDict(const Config & cfg)
+    : WritableBase(replacement_dict, "WritableReplDict", ".prepl",".rpl", cfg) 
   {
     fast_lookup = true;
   }
@@ -817,16 +837,23 @@ PosibErr<void> WritableReplDict::save (FStream & out, ParmString file_name)
 {
   out.printf("personal_repl-1.1 %s 0 %s\n", lang_name(), file_encoding.c_str());
   
-  WordLookup::iterator i = word_lookup->begin();
-  WordLookup::iterator e = word_lookup->end();
+  Vector<const char *> words;
+  words.reserve(word_lookup->size());
+  for (WordLookup::const_iterator i = word_lookup->begin(), e = word_lookup->end();
+       i != e; ++i)
+    words.push_back(*i);
+  std::sort(words.begin(), words.end(), CStrLess());
 
   ConvP conv1(oconv);
   ConvP conv2(oconv);
-  
-  for (;i != e; ++i) 
+
+  Vector<const char *> v;
+  for (Vector<const char *>::const_iterator i = words.begin(), e = words.end();
+       i != e; ++i) 
   {
-    StrVector * v = get_vector(*i);
-    for (StrVector::iterator j = v->begin(); j != v->end(); ++j)
+    v = *get_vector(*i); // make a copy
+    std::sort(v.begin(), v.end(), CStrLess());
+    for (StrVector::iterator j = v.begin(); j != v.end(); ++j)
     {
       write_n_escape(out, conv1(*i));
       out << ' ';
@@ -929,12 +956,12 @@ WritableReplDict::~WritableReplDict()
 
 namespace aspeller {
 
-  Dictionary * new_default_writable_dict() {
-    return new WritableDict();
+  Dictionary * new_default_writable_dict(const Config & cfg) {
+    return new WritableDict(cfg);
   }
 
-  Dictionary * new_default_replacement_dict() {
-    return new WritableReplDict();
+  Dictionary * new_default_replacement_dict(const Config & cfg) {
+    return new WritableReplDict(cfg);
   }
 
 }
