@@ -1,5 +1,5 @@
 // This file is part of The New Aspell Copyright (C)
-// 2002,2003,2004,2011 by Kevin Atkinson under the GNU LGPL license
+// 2002,2003,2004,2011,2019 by Kevin Atkinson under the GNU LGPL license
 // version 2.0 or 2.1.  You should have received a copy of the LGPL
 // license along with this library if you did not you can find it at
 // http://www.gnu.org/.
@@ -27,9 +27,10 @@
 
 #include "aspell.h"
 
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #ifdef USE_FILE_INO
-# include <sys/types.h>
-# include <sys/stat.h>
 # include <unistd.h>
 # include <fcntl.h>
 #endif
@@ -51,6 +52,7 @@
 #include "string_enumeration.hpp"
 #include "string_map.hpp"
 #include "word_list.hpp"
+#include "istream_enumeration.hpp"
 
 #include "string_list.hpp"
 #include "speller_impl.hpp"
@@ -203,10 +205,11 @@ static const ModeAbrv mode_abrvs[] = {
   {'e', "mode=email", N_("enter Email mode.")},
   {'H', "mode=html",  N_("enter HTML mode.")},
   {'t', "mode=tex",   N_("enter TeX mode.")},
-  {'n', "mode=nroff", N_("enter Nroff mode.")}
+  {'n', "mode=nroff", N_("enter Nroff mode.")},
+  {'M', "mode=markdown", N_("enter Markdown mode.")},
 };
 
-static const ModeAbrv *  mode_abrvs_end = mode_abrvs + 4;
+static const ModeAbrv *  mode_abrvs_end = mode_abrvs + sizeof(mode_abrvs)/sizeof(ModeAbrv);
 
 const PossibleOption * find_option(char c) {
   const PossibleOption * i = possible_options;
@@ -234,7 +237,7 @@ static const PossibleOption * find_option(const char * begin, const char * end)
 static const PossibleOption * find_option(const char * str) {
   const PossibleOption * i = possible_options;
   while (i != possible_options_end 
-	 && !strcmp(str, i->name) == 0)
+	 && strcmp(str, i->name) != 0)
     ++i;
   return i;
 }
@@ -252,7 +255,7 @@ Conv uiconv;
 
 int main (int argc, const char *argv[]) 
 {
-  options = new_config(); // this needs to be here becuase of a bug
+  options = new_config(); // this needs to be here because of a bug
                           // with static initlizers on Darwin.
 #ifdef USE_LOCALE
   setlocale (LC_ALL, "");
@@ -358,7 +361,7 @@ int main (int argc, const char *argv[])
     }
   }
 
-  options->read_in_settings();
+  EXIT_ON_ERR(options->read_in_settings());
 
   const char * codeset = 0;
 #ifdef HAVE_LANGINFO_CODESET
@@ -490,7 +493,7 @@ int main (int argc, const char *argv[])
 
   
 static Convert * setup_conv(const aspeller::Language * lang,
-                                      Config * config)
+                            Config * config)
 {
   if (config->retrieve("encoding") != "none") {
     PosibErr<Convert *> pe = new_convert_if_needed(*config,
@@ -503,7 +506,7 @@ static Convert * setup_conv(const aspeller::Language * lang,
     return 0;
   }
 }
- 
+
 static Convert * setup_conv(Config * config,
                             const aspeller::Language * lang)
 {
@@ -674,7 +677,7 @@ void status_fun(void * d, Token, int correct)
     if (ci->compound)
       COUT.put("-\n");
     else if (ci->pre_flag || ci->suf_flag)
-      COUT.printf("+ %s\n", p->oconv(ci->word.str()));
+      COUT.printf("+ %s\n", p->oconv(ci->word.str, ci->word.len));
     else
       COUT.put("*\n");
   }
@@ -754,6 +757,7 @@ void pipe()
       buf.push_back(static_cast<char>(c));
     buf.push_back('\n'); // always add new line so strlen > 0
     buf.push_back('\0');
+    //CERR.printf("%s", buf.data());
     line = buf.data();
     ignore = 0;
     switch (line[0]) {
@@ -872,14 +876,15 @@ void pipe()
           guess.clear();
           if (ci->pre_add && ci->pre_add[0])
             guess.append(ci->pre_add, ci->pre_add_len).append('+');
-          guess.append(ci->word);
+          guess.append(ci->word.str, ci->word.len);
           if (ci->pre_strip_len > 0) 
-            guess.append('-').append(ci->word.str(), ci->pre_strip_len);
+            guess.append('-').append(ci->word.str, ci->pre_strip_len);
           if (ci->suf_strip_len > 0) 
-            guess.append('-').append(ci->word.str() + ci->word.size() - ci->suf_strip_len, 
+            guess.append('-').append(ci->word.str + ci->word.len - ci->suf_strip_len, 
                                      ci->suf_strip_len);
           if (ci->suf_add && ci->suf_add[0])
             guess.append('+').append(ci->suf_add, ci->suf_add_len);
+          guess.ensure_null_end();
           real_speller->lang().fix_case(casep, guess.data(), guess.data());
           guesses << ", " << oconv(guess.str());
           ci = ci->next;
@@ -1006,16 +1011,20 @@ void check()
     exit(1);
   }
 
-#ifdef USE_FILE_INO
   {
     struct stat st;
     fstat(fileno(in), &st);
+    if (!S_ISREG(st.st_mode)) {
+      print_error(_("\"%s\" is not a regular file"), file_name);
+      exit(-1);
+    }
+#ifdef USE_FILE_INO
     int fd = open(new_name.c_str(), O_WRONLY | O_CREAT | O_TRUNC, st.st_mode);
     if (fd >= 0) out = fdopen(fd, "w");
-  }
 #else
-  out = fopen(new_name.c_str(), "w");
+    out = fopen(new_name.c_str(), "w");
 #endif
+  }
   if (!out) {
     print_error(_("Could not open the file \"%s\" for writing. File not saved."), file_name);
     exit(-1);
@@ -1195,7 +1204,7 @@ exit_loop:
   }
 abort_loop:
   {
-    state.del(); // to close the file handles
+    state->abort(); // to close the file handles
     delete_aspell_speller(speller);
 
     remove_file(new_name);
@@ -1381,12 +1390,46 @@ void normlz()
 
 void filter()
 {
-  //assert(setvbuf(stdin, 0, _IOLBF, 0) == 0);
-  //assert(setvbuf(stdout, 0, _IOLBF, 0) == 0);
-  CERR << _("Sorry \"filter\" is currently unimplemented.\n");
-  exit(3);
-}
+  line_buffer();
 
+  String enc = options->retrieve("encoding");
+  if (enc == "none")
+    enc="utf-8";
+  String encoding;
+  fix_encoding_str(enc, encoding);
+
+  EXIT_ON_ERR_SET(Decode::get_new(encoding, options.get()),
+                  StackPtr<Decode>, iconv);
+  EXIT_ON_ERR_SET(Encode::get_new(encoding, options.get()),
+                  StackPtr<Encode>, oconv);
+
+  StackPtr<Filter> filter(new Filter);
+  EXIT_ON_ERR(setup_filter(*filter, options.get(), true, true, false));
+
+  CharVector buf;
+  FilterCharVector proc_str;
+
+  int c;
+  for (;;) {
+    buf.clear();
+    while (c = getchar(), c != '\n' && c != EOF)
+      buf.push_back(static_cast<char>(c));
+    if (c == EOF) break;
+    buf.push_back('\n');
+    buf.push_back('\0');
+    proc_str.clear();
+    iconv->decode(buf.data(), buf.size()-1, proc_str);
+    proc_str.append(0);
+    FilterChar * begin = proc_str.pbegin();
+    FilterChar * end   = proc_str.pend() - 1;
+    filter->process(begin, end);
+    buf.clear();
+    oconv->encode(begin,end,buf);
+    if (buf.empty() || buf.back() != '\n')
+      buf.push_back('\n');
+    COUT.write(buf);
+  }
+}
 
 ///////////////////////////
 //
@@ -1404,24 +1447,6 @@ void print_ver () {
 // module
 //
 
-class IstreamEnumeration : public StringEnumeration {
-  FStream * in;
-  String data;
-public:
-  IstreamEnumeration(FStream & i) : in(&i) {}
-  IstreamEnumeration * clone() const {
-    return new IstreamEnumeration(*this);
-  }
-  void assign (const StringEnumeration * other) {
-    *this = *static_cast<const IstreamEnumeration *>(other);
-  }
-  Value next() {
-    if (!in->getline(data)) return 0;
-    else return data.c_str();
-  }
-  bool at_end() const {return *in;}
-};
-
 ///////////////////////////
 //
 // clean
@@ -1452,10 +1477,10 @@ void clean()
   Conv oconv, oconv2;
   if (config->have("encoding")) {
     EXIT_ON_ERR(oconv.setup(*config, lang->charmap(), config->retrieve("encoding"), NormTo));
-    oconv2.setup(*config, lang->charmap(), config->retrieve("encoding"), NormTo);
+    EXIT_ON_ERR(oconv2.setup(*config, lang->charmap(), config->retrieve("encoding"), NormTo));
   } else {
     EXIT_ON_ERR(oconv.setup(*config, lang->charmap(), lang->data_encoding(), NormTo));
-    oconv2.setup(*config, lang->charmap(), lang->data_encoding(), NormTo);
+    EXIT_ON_ERR(oconv2.setup(*config, lang->charmap(), lang->data_encoding(), NormTo));
   }
   while (wl_itr.adv()) {
     if (*wl_itr->aff.str) 
@@ -1569,8 +1594,8 @@ void personal () {
     // FIXME: This is currently broken
 
     Config * config = options;
-    Dictionary * per = new_default_writable_dict();
-    per->load(config->retrieve("personal-path"), *config);
+    Dictionary * per = new_default_writable_dict(*config);
+    EXIT_ON_ERR(per->load(config->retrieve("personal-path"), *config));
     StackPtr<WordEntryEnumeration> els(per->detailed_elements());
     StackPtr<Convert> conv(setup_conv(per->lang(), config));
 
@@ -1631,7 +1656,7 @@ void repl() {
 
     // FIXME: This is currently broken
 
-    ReplacementDict * repl = new_default_replacement_dict();
+    ReplacementDict * repl = new_default_replacement_dict(*options);
     repl->load(options->retrieve("repl-path"), *options);
     StackPtr<WordEntryEnumeration> els(repl->detailed_elements());
     
@@ -1695,7 +1720,7 @@ void munch()
     COUT << word;
     for (const aspeller::CheckInfo * ci = gi.head; ci; ci = ci->next)
     {
-      COUT << ' ' << oconv(ci->word) << '/';
+      COUT << ' ' << oconv(ci->word.str, ci->word.len) << '/';
       if (ci->pre_flag != 0) COUT << oconv(static_cast<char>(ci->pre_flag));
       if (ci->suf_flag != 0) COUT << oconv(static_cast<char>(ci->suf_flag));
     }
@@ -1966,7 +1991,7 @@ void munch_list_simple()
     while (ci)
     { {
       // check if the base word is in the dictionary
-      SML_Table::iterator b = table.find(ci->word);
+      SML_Table::iterator b = table.find(ci->word.str);
       if (b == table.end()) goto cont;
 
       // check if all the words once expanded are in the dictionary
@@ -1976,15 +2001,15 @@ void munch_list_simple()
       if (ci->pre_flag != 0) flags += ci->pre_flag;
       if (ci->suf_flag != 0) flags += ci->suf_flag;
       exp_buf.reset();
-      exp_list = lang->expand(ci->word, flags, exp_buf);
+      exp_list = lang->expand(ci->word.str, flags, exp_buf);
       for (WordAff * q = exp_list; q; q = q->next) {
         if (!table.have(q->word)) goto cont;
       }
 
       // the base word and flags are valid, now keep the one with the
       // smallest base word
-      if (ci->word.size() < min_base_size) {
-        min_base_size = ci->word.size();
+      if (ci->word.len < min_base_size) {
+        min_base_size = ci->word.len;
         best = ci;
       }
 
@@ -1992,9 +2017,9 @@ void munch_list_simple()
       ci = ci->next;
     }
     // now add the base to the keep list if one exists
-    // otherwise just keep the orignal word
+    // otherwise just keep the original word
     if (best) {
-      SML_Table::iterator b = table.find(best->word);
+      SML_Table::iterator b = table.find(best->word.str);
       assert(b != table.end());
       if (best->pre_flag) add_affix(b, best->pre_flag);
       if (best->suf_flag) add_affix(b, best->suf_flag);
@@ -2327,7 +2352,7 @@ void munch_list_complete(bool multi, bool simplify)
     while (ci)
     { {
       // check if the base word is in the dictionary
-      CML_Table::iterator b = table.find(ci->word);
+      CML_Table::iterator b = table.find(ci->word.str);
       if (b == table.end()) goto cont;
 
       // check if all the words once expanded are in the dictionary
@@ -2337,7 +2362,7 @@ void munch_list_complete(bool multi, bool simplify)
       else if (ci->suf_flag != 0) flags[0] = ci->suf_flag;
       flags[1] = '\0';
       exp_buf.reset();
-      exp_list = lang->expand(ci->word, flags, exp_buf);
+      exp_list = lang->expand(ci->word.str, flags, exp_buf);
       for (WordAff * q = exp_list; q; q = q->next) {
         if (!table.have(q->word)) goto cont;
       }
@@ -2637,7 +2662,7 @@ void munch_list_complete(bool multi, bool simplify)
       if (working.size() > 0 && working.size() == prev_working_size)
       {
         to_keep.push_back(working[0]);
-        //CERR.printf("Making greedy choice! Chosing %s/%s.\n",
+        //CERR.printf("Making greedy choice! Choosing %s/%s.\n",
         //            working[0]->word, working[0]->aff);
         merge(to_keep_exp, working[0]->exp);
         working.erase(working.begin(), working.begin() + 1);
@@ -2785,7 +2810,12 @@ void print_help_line(char abrv, char dont_abrv, const char * name,
   if (type == KeyInfoInt)
     command += "=<int>";
   const char * tdesc = _(desc);
-  printf("  %-27s %s\n", command.c_str(), tdesc); // FIXME: consider word wrapping
+  char buf[120];
+  int len = snprintf(buf, 120, "  %-27s", command.c_str());
+  if (len == 29)
+    printf("%s %s\n", buf, tdesc);
+  else
+    printf("%s\n%30s%s\n", buf, "", tdesc);
 }
 
 namespace acommon {
@@ -2800,7 +2830,7 @@ static const char * usage_text[] =
   N_("<command> is one of:"),
   N_("  -?|usage         display a brief usage message"),
   N_("  help             display a detailed help message"),
-  N_("  -c|check <file>  to check a file"),
+  N_("  -c|check <file>  spellchecks a file"),
   N_("  -a|pipe          \"ispell -a\" compatibility mode"),
   N_("  [dump] config    dumps the current configuration to stdout"),
   N_("  config <key>     prints the current value of an option"),
@@ -2826,7 +2856,7 @@ static const char * help_text[] =
   N_("  munch            generate possible root words and affixes"),
   N_("  expand [1-4]     expands affix flags"),
   N_("  clean [strict]   cleans a word list so that every line is a valid word"),
-  //N_("  filter           passes standard input through filters"),
+  N_("  filter           filters input as if it was being spellchecked"),
   N_("  -v|version       prints a version line"),
   N_("  munch-list [simple] [single|multi] [keep]"),
   N_("    reduce the size of a word list via affix compression"),
@@ -2852,7 +2882,7 @@ void print_help (bool verbose) {
   load_all_filters(options);
   if (verbose) {
     printf(_("\n"
-             "Aspell %s.  Copyright 2000-2011 by Kevin Atkinson.\n"
+             "Aspell %s.  Copyright 2000-2019 by Kevin Atkinson.\n"
              "\n"), aspell_version_string());
     for (unsigned i = 0; i < help_text_size; ++i)
       puts(gt_(help_text[i]));
